// Geometrische Engine fr 3D (gex) V1.0        1998 by M.D.Schiller

// History
// 12. August  -  Zusammenfgen der Routienen aus den bisher erarbeiteten in eine Bibliothek.
// 13. August  -  Get Move Direktion hinzugefgt 

// Strukturen

// 4x4 Matrixstruktur
typedef struct _Matrix3D 
{
    float e[4][4];
} Matrix3D;

// Vertexstruktur
typedef struct _Vertex3D 
{
    float x;
    float y;
    float z;
    float w;
} Vertex3D;


// Globale Variabeln
Matrix3D  matrix_out;
Matrix3D  matrix_globalworld;
Matrix3D  matrix_zero;
Vertex3D  vertex_out;
float gex_xout;
float gex_yout;
float gex_zout;
float gex_flagout;


// Grundlegende Rechenfuktionen

// Matrixmultiplikation
void gex_MatrixMul(Matrix3D a, Matrix3D b)
{
    int i,j,k;
    Matrix3D c;
    for( i= 0; i < 4;i++) {
        for( j = 0; j < 4;j++) {
            c.e[i][j] = 0.0;
            for( k = 0;k < 4; k++ ) c.e[i][j] += a.e[i][k] * b.e[k][j];
        }
    }
    matrix_out = c;
}


// Erstellt die Matrixen die bentigt werden um die Bildschirmpunkte zu ermitteln. 
// Die gex_setmatrixglobalworld Matrixen sind nur fr die Globale Welt gltig und nicht 
// zum einfgen von Objekten in das globale System, da in dieser Matrix die 
// Zentralprojektion enthalten ist. 

void gex_setmatrixglobalworld(float dx,float dy,float dz,               // Translation
                                                float xrot,float yrot,float zrot,        // Rotation
                                                float skax,float skay,float skaz,    // Skalierung
                                                float prod) {                                 // d
    
    Matrix3D erg,tra,rox,roy,roz,pro,ska;
    
    // Skalierungsmatrix
    ska.e[0][0] = skax;   ska.e[1][0] = 0;        ska.e[2][0] = 0;        ska.e[3][0] = 0;
    ska.e[0][1] = 0;        ska.e[1][1] = skay;   ska.e[2][1] = 0;        ska.e[3][1] = 0;
    ska.e[0][2] = 0;        ska.e[1][2] = 0;        ska.e[2][2] = skaz;    ska.e[3][2] = 0;
    ska.e[0][3] = 0;        ska.e[1][3] = 0;        ska.e[2][3] = 0;         ska.e[3][3] = 1;

    // Translationsmatrix 
    tra.e[0][0] = 1;   tra.e[1][0] = 0;   tra.e[2][0] = 0;    tra.e[3][0] = dx;
    tra.e[0][1] = 0;   tra.e[1][1] = 1;   tra.e[2][1] = 0;    tra.e[3][1] = dy;
    tra.e[0][2] = 0;   tra.e[1][2] = 0;   tra.e[2][2] = 1;    tra.e[3][2] = dz;
    tra.e[0][3] = 0;   tra.e[1][3] = 0;   tra.e[2][3] = 0;    tra.e[3][3] = 1;

    // z-Rotationsmatrix
    roz.e[0][0] = cos(zrot);   roz.e[1][0] = -sin(zrot);   roz.e[2][0] = 0;    roz.e[3][0] = 0;
    roz.e[0][1] = sin(zrot);    roz.e[1][1] = cos(zrot);   roz.e[2][1] = 0;    roz.e[3][1] = 0;
    roz.e[0][2] = 0;               roz.e[1][2] = 0;               roz.e[2][2] = 1;    roz.e[3][2] = 0;
    roz.e[0][3] = 0;               roz.e[1][3] = 0;               roz.e[2][3] = 0;    roz.e[3][3] = 1;

    // x-Rotationmatrix
    rox.e[0][0] = 1;               rox.e[1][0] = 0;               rox.e[2][0] = 0;                rox.e[3][0] = 0;
    rox.e[0][1] = 0;               rox.e[1][1] = cos(xrot);   rox.e[2][1] = -sin(xrot);    rox.e[3][1] = 0;
    rox.e[0][2] = 0;               rox.e[1][2] = sin(xrot);    rox.e[2][2] = cos(xrot);    rox.e[3][2] = 0;
    rox.e[0][3] = 0;               rox.e[1][3] = 0;               rox.e[2][3] = 0;               rox.e[3][3] = 1;

    // y- Rotationmatrix
    roy.e[0][0] = cos(yrot);   roy.e[1][0] = 0;               roy.e[2][0] = sin(yrot);    roy.e[3][0] = 0;
    roy.e[0][1] = 0;               roy.e[1][1] = 1;               roy.e[2][1] = 0;               roy.e[3][1] = 0;
    roy.e[0][2] = -sin(yrot);   roy.e[1][2] = 0;               roy.e[2][2] = cos(yrot);   roy.e[3][2] = 0;
    roy.e[0][3] = 0;               roy.e[1][3] = 0;               roy.e[2][3] = 0;               roy.e[3][3] = 1;

    // Projektionsmatrix  - deaktiviert
    pro.e[0][0] = 1;   pro.e[1][0] = 0;               pro.e[2][0] = 0;           pro.e[3][0] = 0;
    pro.e[0][1] = 0;   pro.e[1][1] = 1;               pro.e[2][1] = 0;           pro.e[3][1] = 0;
    pro.e[0][2] = 0;   pro.e[1][2] = 0;               pro.e[2][2] = 1;           pro.e[3][2] = 0;
    pro.e[0][3] = 0;   pro.e[1][3] = 0;               pro.e[2][3] = 1/prod;   pro.e[3][3] = 0;

    /*gex_MatrixMul(tra,roz);
    erg = matrix_out;
    gex_MatrixMul(erg,roy);
    erg = matrix_out;
    gex_MatrixMul(erg,rox);
    matrix_globalworld = matrix_out;*/

    gex_MatrixMul(ska,roz);
    erg = matrix_out;
    gex_MatrixMul(erg,roy);
    erg = matrix_out;
    gex_MatrixMul(erg,rox);
    erg = matrix_out;
    gex_MatrixMul(erg,tra);
    matrix_globalworld = matrix_out;
    
    /*gex_MatrixMul(roz,roy);
    erg = matrix_out;
    gex_MatrixMul(erg,rox);
    erg = matrix_out;
    gex_MatrixMul(erg,tra);
    matrix_globalworld = matrix_out;*/
    
}
                                                                                                         
// Berechnet das Vertex mit hilfe der globalen Matrix
   
void gex_vertexmulmatrix(Vertex3D vertex , Matrix3D matrix) {
    // Matrixen einbeziehen
    vertex_out.x = (vertex.x * matrix.e[0][0]) + (vertex.y * matrix.e[1][0])
                        + (vertex.z * matrix.e[2][0]) + (vertex.w * matrix.e[3][0]);
    vertex_out.y = (vertex.x * matrix.e[0][1]) + (vertex.y * matrix.e[1][1])
                        + (vertex.z * matrix.e[2][1]) + (vertex.w * matrix.e[3][1]);
    vertex_out.z = (vertex.x * matrix.e[0][2]) + (vertex.y * matrix.e[1][2])
                        + (vertex.z * matrix.e[2][2]) + (vertex.w * matrix.e[3][2]);
    vertex_out.w = (vertex.x * matrix.e[0][3]) + (vertex.y * matrix.e[1][3])
                        + (vertex.z * matrix.e[2][3]) + (vertex.w * matrix.e[3][3]);
}

void gex_calculatepoint(Vertex3D vertex) {

    const float vertex_snapper = (float) (3L << 18 );

    float rx;
    float ry;

    gex_vertexmulmatrix(vertex,matrix_globalworld);
        
    gex_zout = vertex_out.z;
    if ((vertex_out.z / 500) > 0) {
        rx = vertex_out.x / (vertex_out.z / 500);
        ry = vertex_out.y / (vertex_out.z / 500);
    }
    gex_xout = rx + (device3d.param.win_xsize/2);
    gex_yout = ry + (device3d.param.win_ysize/2);

    // Vertexsnapper
    gex_xout += vertex_snapper;
    gex_xout -= vertex_snapper;
    gex_yout += vertex_snapper;
    gex_yout -= vertex_snapper;

    // berprfungen ob Punkt im Bildschirm ist
    if (gex_xout < -200) gex_flagout = 1;
    if (gex_xout > (device3d.param.win_xsize+200) ) gex_flagout = 1;
    if (gex_yout < -200) gex_flagout = 1;
    if (gex_yout > (device3d.param.win_ysize+200) ) gex_flagout = 1;
    if (gex_zout < -200) gex_flagout = 1;
}

void gex_calculatepoint_zeromatrix(Vertex3D vertex) {

    const float vertex_snapper = (float) (3L << 18 );

    float rx;
    float ry;

    gex_vertexmulmatrix(vertex,matrix_zero);
        
    gex_zout = vertex_out.z;
    if ((vertex_out.z / 500) > 0) {
        rx = vertex_out.x / (vertex_out.z / 500);
        ry = vertex_out.y / (vertex_out.z / 500);
    }
    gex_xout = rx + (device3d.param.win_xsize/2);
    gex_yout = ry + (device3d.param.win_ysize/2);

    // Vertexsnapper
    gex_xout += vertex_snapper;
    gex_xout -= vertex_snapper;
    gex_yout += vertex_snapper;
    gex_yout -= vertex_snapper;

    // berprfungen ob Punkt im Bildschirm ist
    if (gex_xout < 0) gex_flagout = 1;
    if (gex_xout > (device3d.param.win_xsize)) gex_flagout = 1;
    if (gex_yout < 0) gex_flagout = 1;
    if (gex_yout > (device3d.param.win_ysize)) gex_flagout = 1;
    if (gex_zout < 0) gex_flagout = 1;
}









